@@ -1,31 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2019 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the Qt Shader Tools module
-**
-** $QT_BEGIN_LICENSE:GPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 or (at your option) any later version
-** approved by the KDE Free Qt Foundation. The licenses are as published by
-** the Free Software Foundation and appearing in the file LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2021 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qshaderbaker_p.h"
 #include "qspirvcompiler_p.h"
@@ -529,7 +503,7 @@ QShader QShaderBaker::bake()
 
     // Now spirv, and, if in use, batchableSpirv, contain at least one,
     // optionally more SPIR-V binaries.
-    Q_ASSERT(!spirv.isEmpty() && (d->perTargetEnabled || spirv.count() == 1));
+    Q_ASSERT(!spirv.isEmpty() && (d->perTargetEnabled || spirv.size() == 1));
 
     QShader bs;
     bs.setStage(d->stage);
@@ -591,11 +565,36 @@ QShader QShaderBaker::bake()
                 QSpirvShader::GlslFlags flags;
                 if (req.second.flags().testFlag(QShaderVersion::GlslEs))
                     flags |= QSpirvShader::GlslEs;
-                shader.setShader(currentSpirvShader->translateToGLSL(req.second.version(), flags));
+                QVector<QSpirvShader::SeparateToCombinedImageSamplerMapping> separateToCombinedImageSamplerMappings;
+                shader.setShader(currentSpirvShader->translateToGLSL(req.second.version(), flags, &separateToCombinedImageSamplerMappings));
                 if (shader.shader().isEmpty()) {
                     d->errorMessage = currentSpirvShader->translationErrorMessage();
                     return QShader();
                 }
+                if (!separateToCombinedImageSamplerMappings.isEmpty()) {
+                    const QShaderDescription desc = bs.description();
+                    QVector<QShaderDescription::InOutVariable> separateImages = desc.separateImages();
+                    QVector<QShaderDescription::InOutVariable> separateSamplers = desc.separateSamplers();
+                    QShader::SeparateToCombinedImageSamplerMappingList result;
+                    for (const QSpirvShader::SeparateToCombinedImageSamplerMapping &mapping : separateToCombinedImageSamplerMappings) {
+                        int textureBinding = -1;
+                        int samplerBinding = -1;
+                        for (int i = 0, count = separateImages.size(); i < count; ++i) {
+                            if (separateImages[i].name == mapping.textureName) {
+                                textureBinding = separateImages[i].binding;
+                                break;
+                            }
+                        }
+                        for (int i = 0, count = separateSamplers.size(); i < count; ++i) {
+                            if (separateSamplers[i].name == mapping.samplerName) {
+                                samplerBinding = separateSamplers[i].binding;
+                                break;
+                            }
+                        }
+                        result.append({ mapping.combinedSamplerName, textureBinding, samplerBinding });
+                    }
+                    bs.setSeparateToCombinedImageSamplerMappingList(key, result);
+                }
             }
                 break;
             case QShader::HlslShader:
