@@ -1,30 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2021 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the test suite of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2021 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include <QtTest/QtTest>
 #include <QFile>
@@ -62,6 +37,7 @@ private slots:
     void perTargetCompileMode();
     void serializeDeserialize();
     void spirvOptions();
+    void separateImagesAndSamplers();
 };
 
 void tst_QShaderBaker::initTestCase()
@@ -668,23 +644,23 @@ void tst_QShaderBaker::mslNativeBindingMap()
     QVERIFY(baker.errorMessage().isEmpty());
 
     const QShaderKey mslShaderKey(QShader::MslShader, QShaderVersion(12));
-    QVERIFY(s.nativeResourceBindingMap(mslShaderKey));
+    QVERIFY(!s.nativeResourceBindingMap(mslShaderKey).isEmpty());
 
-    const QShader::NativeResourceBindingMap *nativeBindingMap = s.nativeResourceBindingMap(mslShaderKey);
-    QCOMPARE(nativeBindingMap->count(), 3);
-    QVERIFY(nativeBindingMap->contains(0)); // uniform block
-    QVERIFY(nativeBindingMap->contains(1)); // combined image sampler, maps to a texture and sampler in MSL
-    QVERIFY(nativeBindingMap->contains(2)); // same
+    QShader::NativeResourceBindingMap nativeBindingMap = s.nativeResourceBindingMap(mslShaderKey);
+    QCOMPARE(nativeBindingMap.count(), 3);
+    QVERIFY(nativeBindingMap.contains(0)); // uniform block
+    QVERIFY(nativeBindingMap.contains(1)); // combined image sampler, maps to a texture and sampler in MSL
+    QVERIFY(nativeBindingMap.contains(2)); // same
 
     // MSL has per-resource namespaces
-    QPair<int, int> nativeBindingPair = nativeBindingMap->value(0);
+    QPair<int, int> nativeBindingPair = nativeBindingMap.value(0);
     QCOMPARE(nativeBindingPair.first, 0); // uniform buffer
 
-    nativeBindingPair = nativeBindingMap->value(1);
+    nativeBindingPair = nativeBindingMap.value(1);
     QCOMPARE(nativeBindingPair.first, 0); // texture
     QCOMPARE(nativeBindingPair.second, 0); // sampler
 
-    nativeBindingPair = nativeBindingMap->value(2);
+    nativeBindingPair = nativeBindingMap.value(2);
     QCOMPARE(nativeBindingPair.first, 1); // texture
     QCOMPARE(nativeBindingPair.second, 1); // sampler
 
@@ -696,21 +672,20 @@ void tst_QShaderBaker::mslNativeBindingMap()
     QVERIFY(s.isValid());
     QVERIFY(baker.errorMessage().isEmpty());
 
-    QVERIFY(s.nativeResourceBindingMap(mslShaderKey));
     nativeBindingMap = s.nativeResourceBindingMap(mslShaderKey);
-    QCOMPARE(nativeBindingMap->count(), 8);
+    QCOMPARE(nativeBindingMap.count(), 8);
 
     // rather won't compare with exact values, do not make assumptions, just
     // make sure there is a valid value for each since all resources are used
     // in the shader code
-    QVERIFY(nativeBindingMap->value(1).first != -1);
-    QVERIFY(nativeBindingMap->value(2).first != -1);
-    QVERIFY(nativeBindingMap->value(3).first != -1);
-    QVERIFY(nativeBindingMap->value(4).first != -1);
-    QVERIFY(nativeBindingMap->value(5).first != -1);
-    QVERIFY(nativeBindingMap->value(10).first != -1);
-    QVERIFY(nativeBindingMap->value(11).first != -1);
-    QVERIFY(nativeBindingMap->value(12).first != -1);
+    QVERIFY(nativeBindingMap.value(1).first != -1);
+    QVERIFY(nativeBindingMap.value(2).first != -1);
+    QVERIFY(nativeBindingMap.value(3).first != -1);
+    QVERIFY(nativeBindingMap.value(4).first != -1);
+    QVERIFY(nativeBindingMap.value(5).first != -1);
+    QVERIFY(nativeBindingMap.value(10).first != -1);
+    QVERIFY(nativeBindingMap.value(11).first != -1);
+    QVERIFY(nativeBindingMap.value(12).first != -1);
 }
 
 void tst_QShaderBaker::hlslNativeBindingMap()
@@ -727,36 +702,35 @@ void tst_QShaderBaker::hlslNativeBindingMap()
     QVERIFY(baker.errorMessage().isEmpty());
 
     const QShaderKey shaderKey(QShader::HlslShader, QShaderVersion(50));
-    QVERIFY(s.nativeResourceBindingMap(shaderKey));
-    const QShader::NativeResourceBindingMap *nativeBindingMap = s.nativeResourceBindingMap(shaderKey);
-    QCOMPARE(nativeBindingMap->count(), 8);
+    QShader::NativeResourceBindingMap nativeBindingMap = s.nativeResourceBindingMap(shaderKey);
+    QCOMPARE(nativeBindingMap.count(), 8);
 
     // do not assume anything about the ordering, so verify that we have valid
     // values, not the register binding values themselves
 
     // combined image samplers
-    QVERIFY(nativeBindingMap->value(1).first != -1);
-    QVERIFY(nativeBindingMap->value(1).second == nativeBindingMap->value(1).first);
-    QVERIFY(nativeBindingMap->value(2).first != -1);
-    QVERIFY(nativeBindingMap->value(2).second == nativeBindingMap->value(2).first);
-    QVERIFY(nativeBindingMap->value(3).first != -1);
-    QVERIFY(nativeBindingMap->value(3).second == nativeBindingMap->value(3).first);
+    QVERIFY(nativeBindingMap.value(1).first != -1);
+    QVERIFY(nativeBindingMap.value(1).second == nativeBindingMap.value(1).first);
+    QVERIFY(nativeBindingMap.value(2).first != -1);
+    QVERIFY(nativeBindingMap.value(2).second == nativeBindingMap.value(2).first);
+    QVERIFY(nativeBindingMap.value(3).first != -1);
+    QVERIFY(nativeBindingMap.value(3).second == nativeBindingMap.value(3).first);
 
-    QVERIFY(nativeBindingMap->value(1).first != nativeBindingMap->value(2).first);
-    QVERIFY(nativeBindingMap->value(1).first != nativeBindingMap->value(3).first);
+    QVERIFY(nativeBindingMap.value(1).first != nativeBindingMap.value(2).first);
+    QVERIFY(nativeBindingMap.value(1).first != nativeBindingMap.value(3).first);
 
     // storage images
-    QVERIFY(nativeBindingMap->value(4).first != -1);
-    QVERIFY(nativeBindingMap->value(5).first != -1);
-    QVERIFY(nativeBindingMap->value(4).first != nativeBindingMap->value(5).first);
+    QVERIFY(nativeBindingMap.value(4).first != -1);
+    QVERIFY(nativeBindingMap.value(5).first != -1);
+    QVERIFY(nativeBindingMap.value(4).first != nativeBindingMap.value(5).first);
 
     // storage buffer
-    QVERIFY(nativeBindingMap->value(10).first != -1);
+    QVERIFY(nativeBindingMap.value(10).first != -1);
 
     // uniform buffers
-    QVERIFY(nativeBindingMap->value(11).first != -1);
-    QVERIFY(nativeBindingMap->value(12).first != -1);
-    QVERIFY(nativeBindingMap->value(11).first != nativeBindingMap->value(12).first);
+    QVERIFY(nativeBindingMap.value(11).first != -1);
+    QVERIFY(nativeBindingMap.value(12).first != -1);
+    QVERIFY(nativeBindingMap.value(11).first != nativeBindingMap.value(12).first);
 }
 
 void tst_QShaderBaker::reflectArraysOfSamplers()
@@ -975,5 +949,69 @@ void tst_QShaderBaker::spirvOptions()
     QVERIFY(bin.size() > strippedBin.size());
 }
 
+void tst_QShaderBaker::separateImagesAndSamplers()
+{
+    QShaderBaker baker;
+    baker.setSourceFileName(QLatin1String(":/data/combined_and_separate_samplers.frag"));
+    baker.setGeneratedShaderVariants({ QShader::StandardShader });
+    QList<QShaderBaker::GeneratedShader> targets;
+    targets.append({ QShader::SpirvShader, QShaderVersion(100) });
+    targets.append({ QShader::GlslShader, QShaderVersion(100, QShaderVersion::GlslEs) });
+    targets.append({ QShader::GlslShader, QShaderVersion(330) });
+    targets.append({ QShader::HlslShader, QShaderVersion(50) });
+    targets.append({ QShader::MslShader, QShaderVersion(12) });
+    baker.setGeneratedShaders(targets);
+    QShader s = baker.bake();
+    QVERIFY(s.isValid());
+    QVERIFY(baker.errorMessage().isEmpty());
+
+    for (QShaderKey shaderKey : {
+         QShaderKey(QShader::HlslShader, QShaderVersion(50)),
+         QShaderKey(QShader::MslShader, QShaderVersion(12)) })
+    {
+        QShader::NativeResourceBindingMap nativeBindingMap = s.nativeResourceBindingMap(shaderKey);
+        QCOMPARE(nativeBindingMap.count(), 4);
+
+        // binding 1 is a combined image sampler
+        QVERIFY(nativeBindingMap.value(1).first != -1);
+        QVERIFY(nativeBindingMap.value(1).second != -1);
+
+        // binding 2 is a separate image
+        QVERIFY(nativeBindingMap.value(2).first != -1);
+        // presumably binding 1 got t0, then this is probably t1, the point is that they should be something different
+        QVERIFY(nativeBindingMap.value(2).first != nativeBindingMap.value(1).first);
+
+        // binding 3 is a separate sampler
+        QVERIFY(nativeBindingMap.value(3).first != -1);
+
+        // binding 4 as well
+        QVERIFY(nativeBindingMap.value(4).first != -1);
+        QVERIFY(nativeBindingMap.value(4).first != nativeBindingMap.value(3).first);
+
+        QVERIFY(s.separateToCombinedImageSamplerMappingList(shaderKey).isEmpty());
+    }
+
+    // Now onto to the real special deal here: OpenGL. That has its own mapping
+    // table because SPIRV-Cross auto-generates a "combined image sampler"
+    // (e.g. a sampler2D) for each separate texture+sampler combination.
+    for (QShaderKey shaderKey : {
+         QShaderKey(QShader::GlslShader, QShaderVersion(330)),
+         QShaderKey(QShader::GlslShader, QShaderVersion(100, QShaderVersion::GlslEs)) })
+    {
+        QShader::SeparateToCombinedImageSamplerMappingList list = s.separateToCombinedImageSamplerMappingList(shaderKey);
+        QCOMPARE(list.count(), 2);
+        for (int i = 0; i < list.count(); ++i) {
+            int tbinding = list.at(i).textureBinding;
+            int sbinding = list.at(i).samplerBinding;
+            QCOMPARE(tbinding, 2);
+            QVERIFY(sbinding == 3 || sbinding == 4);
+            QByteArray combinedName = list.at(i).combinedSamplerName;
+            QVERIFY(!combinedName.isEmpty());
+        }
+
+        QVERIFY(s.nativeResourceBindingMap(shaderKey).isEmpty());
+    }
+}
+
 #include <tst_qshaderbaker.moc>
 QTEST_MAIN(tst_QShaderBaker)
